#!/usr/bin/env python
# Generate version information for a program
#
# Copyright (C) 2015  Kevin O'Connor <kevin@koconnor.net>
#
# This file may be distributed under the terms of the GNU GPLv3 license.
import sys, os, subprocess, shlex, time, socket, optparse, logging, traceback

VERSION_FORMAT = """
/* DO NOT EDIT!  This is an autogenerated file.  See scripts/buildversion.py. */
#define BUILD_VERSION "%s"
#define BUILD_TOOLS "%s"
"""

# Run program and return the specified output
def check_output(prog):
    logging.debug("Running %s" % (repr(prog),))
    try:
        process = subprocess.Popen(shlex.split(prog), stdout=subprocess.PIPE)
        output = process.communicate()[0]
        retcode = process.poll()
    except OSError:
        logging.debug("Exception on run: %s" % (traceback.format_exc(),))
        return ""
    logging.debug("Got (code=%s): %s" % (retcode, repr(output)))
    if retcode:
        return ""
    try:
        return output.decode()
    except UnicodeError:
        logging.debug("Exception on decode: %s" % (traceback.format_exc(),))
        return ""

# Obtain version info from "git" program
def git_version():
    if not os.path.exists('.git'):
        logging.debug("No '.git' file/directory found")
        return ""
    ver = check_output("git describe --always --tags --long --dirty").strip()
    logging.debug("Got git version: %s" % (repr(ver),))
    return ver

# Look for version in a ".version" file.  Official release tarballs
# have this file (see scripts/tarball.sh).
def file_version():
    if not os.path.isfile('.version'):
        logging.debug("No '.version' file found")
        return ""
    try:
        f = open('.version', 'r')
        ver = f.readline().strip()
        f.close()
    except OSError:
        logging.debug("Exception on read: %s" % (traceback.format_exc(),))
        return ""
    logging.debug("Got .version: %s" % (repr(ver),))
    return ver

# Generate an output file with the version information
def write_version(outfile, version, toolstr):
    logging.debug("Write file %s and %s" % (repr(version), repr(toolstr)))
    sys.stdout.write("Version: %s\n" % (version,))
    f = open(outfile, 'w')
    f.write(VERSION_FORMAT % (version, toolstr))
    f.close()

# Run "tool --version" for each specified tool and extract versions
def tool_versions(tools):
    tools = [t.strip() for t in tools.split(';')]
    versions = ['', '']
    success = 0
    for tool in tools:
        # Extract first line from "tool --version" output
        verstr = check_output("%s --version" % (tool,)).split('\n')[0]
        # Check if this tool looks like a binutils program
        isbinutils = 0
        if verstr.startswith('GNU '):
            isbinutils = 1
            verstr = verstr[4:]
        # Extract version information and exclude program name
        if ' ' not in verstr:
            continue
        prog, ver = verstr.split(' ', 1)
        if not prog or not ver:
            continue
        # Check for any version conflicts
        if versions[isbinutils] and versions[isbinutils] != ver:
            logging.debug("Mixed version %s vs %s" % (
                repr(versions[isbinutils]), repr(ver)))
            versions[isbinutils] = "mixed"
            continue
        versions[isbinutils] = ver
        success += 1
    cleanbuild = versions[0] and versions[1] and success == len(tools)
    return cleanbuild, "gcc: %s binutils: %s" % (versions[0], versions[1])

def main():
    usage = "%prog [options] <outputheader.h>"
    opts = optparse.OptionParser(usage)
    opts.add_option("-e", "--extra", dest="extra", default="",
                    help="extra version string to append to version")
    opts.add_option("-t", "--tools", dest="tools", default="",
                    help="list of build programs to extract version from")
    opts.add_option("-v", action="store_true", dest="verbose",
                    help="enable debug messages")

    options, args = opts.parse_args()
    if len(args) != 1:
        opts.error("Incorrect arguments")
    outfile = args[0]
    if options.verbose:
        logging.basicConfig(level=logging.DEBUG)

    cleanbuild, toolstr = tool_versions(options.tools)

    ver = git_version()
    cleanbuild = cleanbuild and 'dirty' not in ver
    if not ver:
        ver = file_version()
        # We expect the "extra version" to contain information on the
        # distributor and distribution package version (if
        # applicable).  It is a "clean" build if this is a build from
        # an official release tarball and the above info is present.
        cleanbuild = cleanbuild and ver and options.extra != ""
        if not ver:
            ver = "?"
    if not cleanbuild:
        btime = time.strftime("%Y%m%d_%H%M%S")
        hostname = socket.gethostname()
        ver = "%s-%s-%s" % (ver, btime, hostname)
    write_version(outfile, ver + options.extra, toolstr)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# Script to check a bios image and report info on it.
#
# Copyright (C) 2008  Kevin O'Connor <kevin@koconnor.net>
#
# This file may be distributed under the terms of the GNU GPLv3 license.

import sys, struct
import layoutrom, buildrom

from python23compat import as_bytes

def subst(data, offset, new):
    return data[:offset] + new + data[offset + len(new):]

def checksum(data, start, size, csum):
    sumbyte = buildrom.checksum(data[start:start+size])
    return subst(data, start+csum, sumbyte)

def main():
    # Get args
    objinfo, finalsize, rawfile, outfile = sys.argv[1:]

    # Read in symbols
    objinfofile = open(objinfo, 'r')
    symbols = layoutrom.parseObjDump(objinfofile, 'in')[1]

    # Read in raw file
    f = open(rawfile, 'rb')
    rawdata = f.read()
    f.close()
    datasize = len(rawdata)
    finalsize = int(finalsize) * 1024
    if finalsize == 0:
        finalsize = 64*1024
        if datasize > 64*1024:
            finalsize = 128*1024
            if datasize > 128*1024:
                finalsize = 256*1024
    if datasize > finalsize:
        print("Error!  ROM doesn't fit (%d > %d)" % (datasize, finalsize))
        print("   You have to either increase the size (CONFIG_ROM_SIZE)")
        print("   or turn off some features (such as hardware support not")
        print("   needed) to make it fit.  Trying a more recent gcc version")
        print("   might work too.")
        sys.exit(1)

    # Sanity checks
    start = symbols['code32flat_start'].offset
    end = symbols['code32flat_end'].offset
    expend = layoutrom.BUILD_BIOS_ADDR + layoutrom.BUILD_BIOS_SIZE
    if end != expend:
        print("Error!  Code does not end at 0x%x (got 0x%x)" % (
            expend, end))
        sys.exit(1)
    if datasize > finalsize:
        print("Error!  Code is too big (0x%x vs 0x%x)" % (
            datasize, finalsize))
        sys.exit(1)
    expdatasize = end - start
    if datasize != expdatasize:
        print("Error!  Unknown extra data (0x%x vs 0x%x)" % (
            datasize, expdatasize))
        sys.exit(1)

    # Fix up CSM Compatibility16 table
    if 'csm_compat_table' in symbols and 'entry_csm' in symbols:
        # Field offsets within EFI_COMPATIBILITY16_TABLE
        ENTRY_FIELD_OFS = 14 # Compatibility16CallOffset (UINT16)
        SIZE_FIELD_OFS = 5   # TableLength (UINT8)
        CSUM_FIELD_OFS = 4   # TableChecksum (UINT8)

        tableofs = symbols['csm_compat_table'].offset - symbols['code32flat_start'].offset
        entry_addr = symbols['entry_csm'].offset - layoutrom.BUILD_BIOS_ADDR
        entry_addr = struct.pack('<H', entry_addr)
        rawdata = subst(rawdata, tableofs+ENTRY_FIELD_OFS, entry_addr)

        tsfield = tableofs+SIZE_FIELD_OFS
        tablesize = ord(rawdata[tsfield:tsfield+1])
        rawdata = checksum(rawdata, tableofs, tablesize, CSUM_FIELD_OFS)

    # Print statistics
    runtimesize = end - symbols['code32init_end'].offset
    print("Total size: %d  Fixed: %d  Free: %d (used %.1f%% of %dKiB rom)" % (
        datasize, runtimesize, finalsize - datasize
        , (datasize / float(finalsize)) * 100.0
        , int(finalsize / 1024)))

    # Write final file
    f = open(outfile, 'wb')
    f.write((as_bytes("\0") * (finalsize - datasize)) + rawdata)
    f.close()

if __name__ == '__main__':
    main()



#!/usr/bin/python3
# Copyright (C) 2011 Red Hat, Inc., Michael S. Tsirkin <mst@redhat.com>
#
# This file may be distributed under the terms of the GNU GPLv3 license.

# Read a preprocessed ASL listing and put each ACPI_EXTRACT
# directive in a comment, to make iasl skip it.
# We also put each directive on a new line, the machinery
# in scripts/acpi_extract.py requires this.

import re
import sys
import fileinput

def die(diag):
    sys.stderr.write("Error: %s\n" % (diag))
    sys.exit(1)

# Note: () around pattern make split return matched string as part of list
psplit = re.compile(r''' (
                          \b # At word boundary
                          ACPI_EXTRACT_\w+ # directive
                          \s+ # some whitespace
                          \w+ # array name
                         )''', re.VERBOSE)

lineno = 0
for line in fileinput.input():
    # line number and debug string to output in case of errors
    lineno = lineno + 1
    debug = "input line %d: %s" % (lineno, line.rstrip())

    s = psplit.split(line)
    # The way split works, each odd item is the matching ACPI_EXTRACT directive.
    # Put each in a comment, and on a line by itself.
    for i in range(len(s)):
        if (i % 2):
            sys.stdout.write("\n/* %s */\n" % s[i])
        else:
            sys.stdout.write(s[i])




#!/usr/bin/python
# encoding: UTF-8

#===============================================================================
# Define global imports
#===============================================================================
import os
import re
import sys
import locale
import codecs
import subprocess as sp
from . import constants


#===============================================================================
# Define module information
#===============================================================================
__author__ = constants.__author__
__license__ = constants.__license__
__copyright__ = constants.__copyright__


#===============================================================================
# Define global constants
#===============================================================================
PYTHON3 = constants.PYTHON3
NoneType = type(None)
APP = constants.APP
DIRS = constants.DIRS
ENCS = constants.ENCS
UTILS = constants.UTILS
MODES = constants.MODES
TESTS = constants.TESTS
compiler = constants.compiler
joinpath = constants.joinpath
cleaner = constants.cleaner
string = constants.string
isabs = os.path.isabs
isdir = os.path.isdir
isfile = os.path.isfile
normpath = os.path.normpath
relpath = os.path.relpath


#===============================================================================
# Define GLInfo class
#===============================================================================
class GLInfo(object):
    '''This class is used to get fromatted information about gnulib-tool.
    This information is mainly used in stdout messages, but can be used
    anywhere else. The return values are not the same as for the module,
    but still depends on them.'''

    def __repr__(self):
        '''x.__repr__ <==> repr(x)'''
        result = '<pygnulib.GLInfo %s>' % hex(id(self))
        return(result)

    def package(self):
        '''Return formatted string which contains name of the package.'''
        result = 'GNU gnulib'
        return(result)

    def authors(self):
        '''Return formatted string which contains authors.
        The special __author__ variable is used (type is list).'''
        result = string()
        for item in __author__:
            if item == __author__[-2]:
                result += '%s ' % item
            elif item == __author__[-1]:
                result += 'and %s' % item
            else:
                result += '%s, ' % item
        return(result)

    def license(self):
        '''Return formatted string which contains license and its description.'''
        result = 'License GPLv3+: GNU GPL version 3 or later'
        result += ' <https://gnu.org/licenses/gpl.html>\n'
        result += 'This is free software: you are free'
        result += ' to change and redistribute it.\n'
        result += 'There is NO WARRANTY, to the extent permitted by law.'
        return(result)

    def copyright(self):
        '''Return formatted string which contains copyright.
        The special __copyright__ variable is used (type is str).'''
        result = 'Copyright (C) %s' % __copyright__
        return(result)

    def date(self):
        '''Return formatted string which contains date and time in GMT format.'''
        if isdir(DIRS['git']):
            counter = int()  # Create counter
            result = string()  # Create string
            args = ['git', 'log']
            result = sp.check_output(args).decode("UTF-8")
            # Get date as "Fri Mar 21 07:16:51 2008 -0600" from string
            pattern = re.compile('Date:[\t ]*(.*?)$', re.S | re.M)
            result = pattern.findall(result)[0]
            # Turn "Fri Mar 21 07:16:51 2008 -0600" into "Mar 21 2008 07:16:51 -0600"
            pattern = re.compile('^[^ ]* ([^ ]*) ([0-9]*) ([0-9:]*) ([0-9]*) ')
            result = pattern.sub('\\1 \\2 \\4 \\3 ', result)
            # Use GNU date to compute the time in GMT
            args = ['date', '-d', result, '-u', '+%Y-%m-%d %H:%M:%S']
            proc = sp.check_output(args)
            result = string(proc, "UTF-8")
            result = result.rstrip(os.linesep)
            return(result)

    def usage(self):
        '''Show help message.'''
        result = '''\
Usage: gnulib-tool --list
       gnulib-tool --find filename
       gnulib-tool --import [module1 ... moduleN]
       gnulib-tool --add-import [module1 ... moduleN]
       gnulib-tool --remove-import [module1 ... moduleN]
       gnulib-tool --update
       gnulib-tool --create-testdir --dir=directory [module1 ... moduleN]
       gnulib-tool --create-megatestdir --dir=directory [module1 ... moduleN]
       gnulib-tool --test --dir=directory module1 ... moduleN
       gnulib-tool --megatest --dir=directory [module1 ... moduleN]
       gnulib-tool --extract-description module
       gnulib-tool --extract-comment module
       gnulib-tool --extract-status module
       gnulib-tool --extract-notice module
       gnulib-tool --extract-applicability module
       gnulib-tool --extract-filelist module
       gnulib-tool --extract-dependencies module
       gnulib-tool --extract-autoconf-snippet module
       gnulib-tool --extract-automake-snippet module
       gnulib-tool --extract-include-directive module
       gnulib-tool --extract-link-directive module
       gnulib-tool --extract-license module
       gnulib-tool --extract-maintainer module
       gnulib-tool --extract-tests-module module
       gnulib-tool --copy-file file [destination]

Operation modes:

      --list                print the available module names
      --find                find the modules which contain the specified file
      --import              import the given modules into the current package
      --add-import          augment the list of imports from gnulib into the
                            current package, by adding the given modules;
                            if no modules are specified, update the current
                            package from the current gnulib
      --remove-import       reduce the list of imports from gnulib into the
                            current package, by removing the given modules
      --update              update the current package, restore files omitted
                            from version control
      --create-testdir      create a scratch package with the given modules
      --create-megatestdir  create a mega scratch package with the given modules
                            one by one and all together
      --test                test the combination of the given modules
                            (recommended to use CC=\"gcc -Wall\" here)
      --megatest            test the given modules one by one and all together
                            (recommended to use CC=\"gcc -Wall\" here)
      --extract-description        extract the description
      --extract-comment            extract the comment
      --extract-status             extract the status (obsolete etc.)
      --extract-notice             extract the notice or banner
      --extract-applicability      extract the applicability
      --extract-filelist           extract the list of files
      --extract-dependencies       extract the dependencies
      --extract-autoconf-snippet   extract the snippet for configure.ac
      --extract-automake-snippet   extract the snippet for library makefile
      --extract-include-directive  extract the #include directive
      --extract-link-directive     extract the linker directive
      --extract-license            report the license terms of the source files
                                   under lib/
      --extract-maintainer         report the maintainer(s) inside gnulib
      --extract-tests-module       report the unit test module, if it exists
      --copy-file                  copy a file that is not part of any module
      --help                Show this help text.
      --version             Show version and authorship information.

General options:

      --dir=DIRECTORY       Specify the target directory.
                            For --import, this specifies where your
                            configure.ac can be found.  Defaults to current
                            directory.
      --local-dir=DIRECTORY  Specify a local override directory where to look
                            up files before looking in gnulib's directory.
      --cache-modules       Enable module caching optimization.
      --no-cache-modules    Disable module caching optimization.
      --verbose             Increase verbosity. May be repeated.
      --quiet               Decrease verbosity. May be repeated.

Options for --import, --add/remove-import, --update:

      --dry-run             Only print what would have been done.

Options for --import, --add/remove-import:

      --with-tests          Include unit tests for the included modules.

Options for --create-[mega]testdir, --[mega]test:

      --without-tests       Don't include unit tests for the included modules.

Options for --import, --add/remove-import,
            --create-[mega]testdir, --[mega]test:

      --with-obsolete       Include obsolete modules when they occur among the
                            dependencies. By default, dependencies to obsolete
                            modules are ignored.
      --with-c++-tests      Include even unit tests for C++ interoperability.
      --without-c++-tests   Exclude unit tests for C++ interoperability.
      --with-longrunning-tests
                            Include even unit tests that are long-runners.
      --without-longrunning-tests
                            Exclude unit tests that are long-runners.
      --with-privileged-tests
                            Include even unit tests that require root
                            privileges.
      --without-privileged-tests
                            Exclude unit tests that require root privileges.
      --with-unportable-tests
                            Include even unit tests that fail on some platforms.
      --without-unportable-tests
                            Exclude unit tests that fail on some platforms.
      --with-all-tests      Include all kinds of problematic unit tests.
      --avoid=MODULE        Avoid including the given MODULE. Useful if you
                            have code that provides equivalent functionality.
                            This option can be repeated.
      --conditional-dependencies
                            Support conditional dependencies (may save configure
                            time and object code).
      --no-conditional-dependencies
                            Don't use conditional dependencies.
      --libtool             Use libtool rules.
      --no-libtool          Don't use libtool rules.

Options for --import, --add/remove-import:

      --lib=LIBRARY         Specify the library name.  Defaults to 'libgnu'.
      --source-base=DIRECTORY
                            Directory relative to --dir where source code is
                            placed (default \"lib\").
      --m4-base=DIRECTORY   Directory relative to --dir where *.m4 macros are
                            placed (default \"m4\").
      --po-base=DIRECTORY   Directory relative to --dir where *.po files are
                            placed (default \"po\").
      --doc-base=DIRECTORY  Directory relative to --dir where doc files are
                            placed (default \"doc\").
      --tests-base=DIRECTORY
                            Directory relative to --dir where unit tests are
                            placed (default \"tests\").
      --aux-dir=DIRECTORY   Directory relative to --dir where auxiliary build
                            tools are placed (default comes from configure.ac).
      --lgpl[=2|=3]         Abort if modules aren't available under the LGPL.
                            Also modify license template from GPL to LGPL.
                            The version number of the LGPL can be specified;
                            the default is currently LGPLv3.
      --makefile-name=NAME  Name of makefile in automake syntax in the
                            source-base and tests-base directories
                            (default \"Makefile.am\").
      --macro-prefix=PREFIX  Specify the prefix of the macros 'gl_EARLY' and
                            'gl_INIT'. Default is 'gl'.
      --po-domain=NAME      Specify the prefix of the i18n domain. Usually use
                            the package name. A suffix '-gnulib' is appended.
      --witness-c-macro=NAME  Specify the C macro that is defined when the
                            sources in this directory are compiled or used.
      --vc-files            Update version control related files.
      --no-vc-files         Don't update version control related files
                            (.gitignore and/or .cvsignore).

Options for --create-[mega]testdir, --[mega]test:

      --single-configure    Generate a single configure file, not a separate
                            configure file for the tests directory.

Options for --import, --add/remove-import, --update,
            --create-[mega]testdir, --[mega]test:

  -s, --symbolic, --symlink Make symbolic links instead of copying files.
      --local-symlink       Make symbolic links instead of copying files, only
                            for files from the local override directory.

Options for --import, --add/remove-import, --update:

  -S, --more-symlinks       Make symbolic links instead of copying files, and
                            don't replace copyright notices.

Report bugs to <bug-gnulib@gnu.org>.'''
        return(result)

    def version(self):
        '''Return formatted string which contains git version.'''
        if isdir(DIRS['git']):
            version_gen = joinpath(DIRS['build-aux'], 'git-version-gen')
            args = [version_gen, DIRS['root']]
            result = sp.check_output(args).decode("UTF-8")
            result = result.strip()
            if result == 'UNKNOWN':
                result = string()
            return(result)



#!/usr/bin/python
# encoding: UTF-8

'''An easy access to pygnulib classes.'''

#===============================================================================
# Define global imports
#===============================================================================
__all__ = list()

try:
    # Constants
    from . import constants

    # Main classes
    from .GLConfig import GLConfig
    from .GLError import GLError
    from .GLInfo import GLInfo

    # File system
    from .GLFileSystem import GLFileSystem
    from .GLFileSystem import GLFileAssistant

    # Module system
    from .GLModuleSystem import GLModule
    from .GLModuleSystem import GLModuleSystem
    from .GLModuleSystem import GLModuleTable

    # Different modes
    from .GLImport import GLImport
    from .GLEmiter import GLEmiter
    from .GLTestDir import GLTestDir
    from .GLTestDir import GLMegaTestDir

    # Other modules
    from .GLMakefileTable import GLMakefileTable
except ValueError as error:
    # Constants
    import constants

    # Main classes
    from GLConfig import GLConfig
    from GLError import GLError
    from GLInfo import GLInfo

    # File system
    from GLFileSystem import GLFileSystem
    from GLFileSystem import GLFileAssistant

    # Module system
    from GLModuleSystem import GLModule
    from GLModuleSystem import GLModuleSystem
    from GLModuleSystem import GLModuleTable

    # Different modes
    from GLImport import GLImport
    from GLEmiter import GLEmiter
    from GLTestDir import GLTestDir
    from GLTestDir import GLMegaTestDir

    # Other modules
    from GLMakefileTable import GLMakefileTable

# Append modules to namespace.
__all__ += ['GLConfig', 'GLError', 'GLInfo']
__all__ += ['GLFileSystem', 'GLFileAssistant']
__all__ += ['GLModule', 'GLModuleSystem', 'GLModuleTable']
__all__ += ['GLImport', 'GLEmiter', 'GLTestDir']
__all__ += ['GLMakefileTable']

#===============================================================================
# Define module information
#===============================================================================
__author__ = constants.__author__
__license__ = constants.__license__
__copyright__ = constants.__copyright__



#!/usr/bin/python
# encoding: UTF-8

#===============================================================================
# Define global imports
#===============================================================================
import os
import re
import sys
import locale
import codecs
import shutil
import filecmp
import subprocess as sp
from . import constants
from .GLError import GLError
from .GLConfig import GLConfig
from .GLModuleSystem import GLModule
from .GLModuleSystem import GLModuleTable
from .GLModuleSystem import GLModuleSystem
from .GLFileSystem import GLFileSystem
from .GLFileSystem import GLFileAssistant
from .GLMakefileTable import GLMakefileTable
from .GLEmiter import GLEmiter


#===============================================================================
# Define module information
#===============================================================================
__author__ = constants.__author__
__license__ = constants.__license__
__copyright__ = constants.__copyright__


#===============================================================================
# Define global constants
#===============================================================================
PYTHON3 = constants.PYTHON3
NoneType = type(None)
APP = constants.APP
DIRS = constants.DIRS
ENCS = constants.ENCS
UTILS = constants.UTILS
MODES = constants.MODES
TESTS = constants.TESTS
compiler = constants.compiler
joinpath = constants.joinpath
cleaner = constants.cleaner
relpath = constants.relativize
string = constants.string
isabs = os.path.isabs
isdir = os.path.isdir
isfile = os.path.isfile
normpath = os.path.normpath


#===============================================================================
# Define GLTestDir class
#===============================================================================
class GLTestDir(object):
    '''GLTestDir class is used to create a scratch package with the given
    list of the modules.'''

    def __init__(self, config, testdir):
        '''GLTestDir.__init__(config, testdir) -> GLTestDir

        Create new GLTestDir instance.'''
        if type(config) is not GLConfig:
            raise(TypeError('config must be a GLConfig, not %s' %
                            type(config).__name__))
        if type(testdir) is bytes or type(testdir) is string:
            if type(testdir) is bytes:
                testdir = testdir.decode(ENCS['default'])
        self.config = config
        self.testdir = os.path.normpath(testdir)
        if not os.path.exists(self.testdir):
            try:  # Try to create directory
                os.mkdir(self.testdir)
            except Exception as error:
                raise(GLError(19, self.testdir))
        self.emiter = GLEmiter(self.config)
        self.filesystem = GLFileSystem(self.config)
        self.modulesystem = GLModuleSystem(self.config)
        self.moduletable = GLModuleTable(self.config)
        self.assistant = GLFileAssistant(self.config)
        self.makefiletable = GLMakefileTable(self.config)

        # Subdirectory names.
        self.config.setSourceBase('gllib')
        self.config.setM4Base('glm4')
        self.config.setDocBase('gldoc')
        self.config.setTestsBase('gltests')
        self.config.setMacroPrefix('gl')
        self.config.resetPoBase()
        self.config.resetPoDomain()
        self.config.resetWitnessCMacro()
        self.config.resetVCFiles()

    def rewrite_files(self, files):
        '''GLTestDir.rewrite_files(files)

        Replace auxdir, docbase, sourcebase, m4base and testsbase from default
        to their version from config.'''
        if type(files) is not list:
            raise(TypeError(
                'files argument must has list type, not %s' % type(files).__name__))
        files = \
            [  # Begin to convert bytes to string
                file.decode(ENCS['default']) \
                if type(file) is bytes else file \
                for file in files
            ]  # Finish to convert bytes to string
        for file in files:
            if type(file) is not string:
                raise(TypeError('each file must be a string instance'))
        files = sorted(set(files))
        auxdir = self.config['auxdir']
        docbase = self.config['docbase']
        sourcebase = self.config['sourcebase']
        m4base = self.config['m4base']
        testsbase = self.config['testsbase']
        result = list()
        for file in files:
            if file.startswith('build-aux/'):
                path = constants.substart('build-aux/', '%s/' % auxdir, file)
            elif file.startswith('doc/'):
                path = constants.substart('doc/', '%s/' % docbase, file)
            elif file.startswith('lib/'):
                path = constants.substart('lib/', '%s/' % sourcebase, file)
            elif file.startswith('m4/'):
                path = constants.substart('m4/', '%s/' % m4base, file)
            elif file.startswith('tests/'):
                path = constants.substart('tests/', '%s/' % testsbase, file)
            elif file.startswith('tests=lib/'):
                path = constants.substart(
                    'tests=lib/', '%s/' % testsbase, file)
            elif file.startswith('top/'):
                path = constants.substart('top/', '', file)
            else:  # file is not a special file
                path = file
            result += [os.path.normpath(path)]
        result = sorted(set(result))
        return(list(result))

    def execute(self):
        '''GLTestDir.execute()

        Create a scratch package with the given modules.'''
        localdir = self.config['localdir']
        auxdir = self.config['auxdir']
        testflags = list(self.config['testflags'])
        sourcebase = self.config['sourcebase']
        m4base = self.config['m4base']
        pobase = self.config['pobase']
        docbase = self.config['docbase']
        testsbase = self.config['testsbase']
        libname = self.config['libname']
        libtool = self.config['libtool']
        witness_c_macro = self.config['witness_c_macro']
        symbolic = self.config['symbolic']
        lsymbolic = self.config['lsymbolic']
        single_configure = self.config['single_configure']
        include_guard_prefix = self.config['include_guard_prefix']
        macro_prefix = self.config['macro_prefix']
        verbose = self.config['verbosity']

        base_modules = [self.modulesystem.find(
            m) for m in self.config['modules']]
        if not base_modules:
            base_modules = self.modulesystem.list()
            base_modules = [self.modulesystem.find(m) for m in base_modules]
        # All modules together.
        # Except config-h, which breaks all modules which use HAVE_CONFIG_H.
        # Except ftruncate, mountlist, which abort the configuration on mingw.
        # Except lib-ignore, which leads to link errors when Sun C++ is used.
        base_modules = sorted(set(base_modules))
        base_modules = [module for module in base_modules if str(module) not in
                        ['config-h', 'ftruncate', 'mountlist', 'lib-ignore']]

        # When computing transitive closures, don't consider $module to depend on
        # $module-tests. Need this because tests are implicitly GPL and may depend
        # on GPL modules - therefore we don't want a warning in this case.
        saved_testflags = list(self.config['testflags'])
        self.config.disableTestFlag(TESTS['tests'])
        for requested_module in base_modules:
            requested_licence = requested_module.getLicense()
            # Here we use self.moduletable.transitive_closure([module]), not just
            # module.getDependencies, so that we also detect weird situations like
            # an LGPL module which depends on a GPLed build tool module which depends
            # on a GPL module.
            if requested_licence != 'GPL':
                modules = self.moduletable.transitive_closure(
                    [requested_module])
                for module in modules:
                    license = module.getLicense()
                    errormsg = 'module %s depends on a module ' % requested_module
                    errormsg += 'with an incompatible license: %s\n' % module
                    if requested_licence == 'GPLv2+':
                        if license not in ['GPLv2+', 'LGPLv2+']:
                            sys.stderr.write(errormsg)
                    elif requested_licence in ['LGPL']:
                        if license not in ['LGPL', 'LGPLv2+']:
                            sys.stderr.write(errormsg)
                    elif requested_licence in ['LGPLv2+']:
                        if license not in ['LGPLv2+']:
                            sys.stderr.write(errormsg)
        self.config.setTestFlags(saved_testflags)

        # Determine final module list.
        modules = self.moduletable.transitive_closure(base_modules)
        final_modules = list(modules)

        # Show final module list.
        if verbose >= 0:
            bold_on = ''
            bold_off = ''
            term = os.getenv('TERM')
            if term == 'xterm':
                bold_on = '\x1b[1m'
                bold_off = '\x1b[0m'
            print('Module list with included dependencies (indented):')
            for module in final_modules:
                if str(module) in self.config.getModules():
                    print('  %s%s%s' % (bold_on, module, bold_off))
                else:  # if str(module) not in self.config.getModules()
                    print('    %s' % module)

        # Generate lists of the modules.
        if single_configure:
            # Determine main module list and tests-related module list separately.
            main_modules, tests_modules = \
                self.moduletable.transitive_closure_separately(
                    base_modules, final_modules)
            # Print main_modules and tests_modules.
            if verbose >= 1:
                print('Main module list:')
                for module in main_modules:
                    print('  %s' % str(module))
                print('Tests-related module list:')
                for module in tests_modules:
                    print('  %s' % str(module))
            # Determine whether a $testsbase/libtests.a is needed.
            libtests = False
            for module in tests_modules:
                files = module.getFiles()
                for file in files:
                    if file.startswith('lib/'):
                        libtests = True
                        break
            if libtests:
                self.config.enableLibtests()

        if single_configure:
            # Add dummy package if it is needed.
            main_modules = self.moduletable.add_dummy(main_modules)
            if 'dummy' in [str(module) for module in main_modules]:
                main_modules = [m for m in main_modules if str(m) != 'dummy']
                dummy = self.modulesystem.find('dummy')
                main_modules = sorted(set(main_modules)) + [dummy]
            if libtests:  # if we need to use libtests.a
                tests_modules = self.moduletable.add_dummy(tests_modules)
                if 'dummy' in [str(module) for module in tests_modules]:
                    tests_modules = [
                        m for m in tests_modules if str(m) != 'dummy']
                    dummy = self.modulesystem.find('dummy')
                    tests_modules = sorted(set(tests_modules)) + [dummy]
        else:  # if not single_configure
            modules = self.moduletable.add_dummy(modules)
            if 'dummy' in [str(module) for module in modules]:
                modules = [m for m in modules if str(m) != 'dummy']
                dummy = self.modulesystem.find('dummy')
                modules = sorted(set(modules)) + [dummy]

        # Show banner notice of every module.
        if single_configure:
            for module in main_modules:
                notice = module.getNotice()
                if notice:
                    print('Notice from module %s:' % str(module))
                    pattern = compiler('^(.*?)$', re.S | re.M)
                    notice = pattern.sub('  \\1', notice)
                    print(notice)
        else:  # if not single_configure
            for module in modules:
                notice = module.getNotice()
                if notice:
                    print('Notice from module %s:' % str(module))
                    pattern = compiler('^(.*?)$', re.S | re.M)
                    notice = pattern.sub('  \\1', notice)
                    print(notice)

        # Determine final file list.
        if single_configure:
            main_filelist, tests_filelist = \
                self.moduletable.filelist_separately(
                    main_modules, tests_modules)
            filelist = sorted(set(main_filelist + tests_filelist))
        else:  # if not single_configure
            filelist = self.moduletable.filelist(modules)

        filelist = sorted(set(filelist))

        # Print list of files.
        if verbose >= 0:
            print('File list:')
            for file in filelist:
                if file.startswith('tests=lib/'):
                    rest = file[10:]
                    print('  lib/%s -> tests/%s' % (rest, rest))
                else:
                    print('  %s' % file)

        # Add files for which the copy in gnulib is newer than the one that
        # "automake --add-missing --copy" would provide.
        filelist += ['build-aux/config.guess', 'build-aux/config.sub']
        filelist = sorted(set(filelist))

        # Create directories.
        directories = [os.path.dirname(file)
                       for file in self.rewrite_files(filelist)]
        directories = sorted(set(directories))

        # Copy files or make symbolic links.
        filetable = list()
        for src in filelist:
            dest = self.rewrite_files([src])[-1]
            filetable += [tuple([dest, src])]
        for row in filetable:
            src = row[1]
            dest = row[0]
            destpath = joinpath(self.testdir, dest)
            dirname = os.path.dirname(destpath)
            if not isdir(dirname):
                os.makedirs(dirname)
            if src.startswith('tests=lib/'):
                src = constants.substart('tests=lib/', 'lib/', src)
            lookedup, flag = self.filesystem.lookup(src)
            if isfile(destpath):
                os.remove(destpath)
            if flag:
                shutil.copy(lookedup, destpath)
            else:  # if not flag
                if symbolic or (lsymbolic and lookedup == joinpath(localdir, src)):
                    constants.link_relative(lookedup, destpath)
                else:
                    shutil.copy(lookedup, destpath)

        # Create $sourcebase/Makefile.am.
        for_test = True
        directory = joinpath(self.testdir, sourcebase)
        if not isdir(directory):
            os.mkdir(directory)
        destfile = joinpath(directory, 'Makefile.am')
        if single_configure:
            emit, uses_subdirs = self.emiter.lib_Makefile_am(destfile, main_modules,
                                                             self.moduletable, self.makefiletable, '', for_test)
        else:  # if not single_configure
            emit, uses_subdirs = self.emiter.lib_Makefile_am(destfile, modules,
                                                             self.moduletable, self.makefiletable, '', for_test)
        with codecs.open(destfile, 'wb', 'UTF-8') as file:
            file.write(emit)
        any_uses_subdirs = uses_subdirs

        # Create $m4base/Makefile.am.
        directory = joinpath(self.testdir, m4base)
        if not isdir(directory):
            os.mkdir(directory)
        destfile = joinpath(directory, 'Makefile.am')
        emit = string()
        emit += '## Process this file with automake to produce Makefile.in.\n\n'
        emit += 'EXTRA_DIST =\n'
        for file in filelist:
            if file.startswith('m4/'):
                file = constants.substart('m4/', '', file)
                emit += 'EXTRA_DIST += %s\n' % file
        emit = constants.nlconvert(emit)
        if type(emit) is bytes:
            emit = emit.decode(ENCS['default'])
        with codecs.open(destfile, 'wb', 'UTF-8') as file:
            file.write(emit)

        subdirs = [sourcebase, m4base]
        subdirs_with_configure_ac = list()

        testsbase_appened = False
        inctests = self.config.checkTestFlag(TESTS['tests'])
        if inctests:
            directory = joinpath(self.testdir, testsbase)
            if not isdir(directory):
                os.mkdir(directory)
            if single_configure:
                # Create $testsbase/Makefile.am.
                destfile = joinpath(directory, 'Makefile.am')
                print(repr(destfile))
                witness_macro = '%stests_WITNESS' % macro_prefix
                emit, uses_subdirs = self.emiter.tests_Makefile_am(destfile,
                                                                   tests_modules, self.makefiletable, witness_macro, for_test)
                with codecs.open(destfile, 'wb', 'UTF-8') as file:
                    file.write(emit)
            else:  # if not single_configure
                # Create $testsbase/Makefile.am.
                destfile = joinpath(directory, 'Makefile.am')
                libtests = False
                self.config.disableLibtests()
                emit, uses_subdirs = self.emiter.tests_Makefile_am(destfile,
                                                                   modules, self.makefiletable, '', for_test)
                with codecs.open(destfile, 'wb', 'UTF-8') as file:
                    file.write(emit)
                # Viewed from the $testsbase subdirectory, $auxdir is different.
                emit = string()
                saved_auxdir = self.config['auxdir']
                testsbase = '%s/' % os.path.normpath(testsbase)
                counter = int()
                auxdir = string()
                finish = (len(testsbase.split('/')) - 1)
                while counter < finish:
                    auxdir += '../'
                    counter += 1
                auxdir = os.path.normpath(joinpath(auxdir, saved_auxdir))
                testsbase = os.path.normpath(testsbase)
                self.config.setAuxDir(auxdir)
                # Create $testsbase/configure.ac.
                emit += '# Process this file with autoconf '
                emit += 'to produce a configure script.\n'
                emit += 'AC_INIT([dummy], [0])\n'
                emit += 'AC_CONFIG_AUX_DIR([%s])\n' % auxdir
                emit += 'AM_INIT_AUTOMAKE\n\n'
                emit += 'AC_CONFIG_HEADERS([config.h])\n\n'
                emit += 'AC_PROG_CC\n'
                emit += 'AC_PROG_INSTALL\n'
                emit += 'AC_PROG_MAKE_SET\n'
                emit += 'gl_PROG_AR_RANLIB\n\n'
                if uses_subdirs:
                    emit += 'AM_PROG_CC_C_O\n\n'
                snippets = list()
                for module in modules:
                    if str(module) in ['gnumakefile', 'maintainer-makefile']:
                        # These are meant to be used only in the top-level directory.
                        pass
                    # if str(module) not in ['gnumakefile', 'maintainer-makefile']
                    else:
                        snippet = module.getAutoconfSnippet_Early()
                        lines = [line for line in snippet.split(
                            '\n') if line.strip()]
                        snippet = '\n'.join(lines)
                        pattern = compiler(
                            'AC_REQUIRE\\(\\[([^()].*?)\\]\\)', re.S | re.M)
                        snippet = pattern.sub('\\1', snippet)
                        snippet = snippet.strip()
                        snippets += [snippet]
                snippets = [snippet for snippet in snippets if snippet.strip()]
                emit += '%s\n' % '\n'.join(snippets)
                if libtool:
                    emit += 'LT_INIT([win32-dll])\n'
                    emit += 'LT_LANG([C++])\n'
                    emit += 'AM_CONDITIONAL([GL_COND_LIBTOOL], [true])\n'
                    emit += 'gl_cond_libtool=true\n'
                else:  # if not libtool
                    emit += 'AM_CONDITIONAL([GL_COND_LIBTOOL], [false])\n'
                    emit += 'gl_cond_libtool=false\n'
                    emit += 'gl_libdeps=\n'
                    emit += 'gl_ltlibdeps=\n'
                # Wrap the set of autoconf snippets into an autoconf macro that is then
                # invoked. This is needed because autoconf does not support AC_REQUIRE
                # at the top level:
                #   error: AC_REQUIRE(gt_CSHARPCOMP): cannot be used outside of an
                #     AC_DEFUN'd macro
                # but we want the AC_REQUIRE to have its normal meaning (provide one
                # expansion of the required macro before the current point, and only
                # one expansion total).
                emit += 'AC_DEFUN([gl_INIT], [\n'
                replace_auxdir = True
                emit += "gl_m4_base='../%s'\n" % m4base
                emit += self.emiter.initmacro_start(macro_prefix)
                # We don't have explicit ordering constraints between the various
                # autoconf snippets. It's cleanest to put those of the library before
                # those of the tests.
                emit += "gl_source_base='../%s'\n" % sourcebase
                emit += self.emiter.autoconfSnippets(modules,
                                                     self.moduletable, self.assistant, 1, False, False, False,
                                                     replace_auxdir)
                emit += "gl_source_base='.'"
                emit += self.emiter.autoconfSnippets(modules,
                                                     self.moduletable, self.assistant, 2, False, False, False,
                                                     replace_auxdir)
                emit += self.emiter.initmacro_end(macro_prefix)
                # _LIBDEPS and _LTLIBDEPS variables are not needed if this library is
                # created using libtool, because libtool already handles the
                # dependencies.
                if not libtool:
                    libname_upper = libname.upper().replace('-', '_')
                    emit += '  %s_LIBDEPS="$gl_libdeps"\n' % libname_upper
                    emit += '  AC_SUBST([%s_LIBDEPS])\n' % libname_upper
                    emit += '  %s_LTLIBDEPS="$gl_ltlibdeps"\n' % libname_upper
                    emit += '  AC_SUBST([%s_LTLIBDEPS])\n' % libname_upper
                emit += '])\n'
                # FIXME use $sourcebase or $testsbase?
                emit += self.emiter.initmacro_done(macro_prefix, sourcebase)
                emit += '\ngl_INIT\n\n'
                # Usually $testsbase/config.h will be a superset of config.h. Verify
                # this by "merging" config.h into $testsbase/config.h; look out for gcc
                # warnings.
                emit += 'AH_TOP([#include \"../config.h\"])\n\n'
                emit += 'AC_CONFIG_FILES([Makefile])\n'
                emit += 'AC_OUTPUT\n'
                emit = constants.nlconvert(emit)
                if type(emit) is bytes:
                    emit = emit.decode(ENCS['default'])
                path = joinpath(self.testdir, testsbase, 'configure.ac')
                with codecs.open(path, 'wb', 'UTF-8') as file:
                    file.write(emit)

                # Restore changed variables.
                self.config.setAuxDir(saved_auxdir)
                auxdir = self.config['auxdir']
                subdirs_with_configure_ac += [testsbase]

            subdirs += [testsbase]
            testsbase_appened = True

        # Create Makefile.am.
        emit = string()
        emit += '## Process this file with automake to produce Makefile.in.\n\n'
        emit += 'AUTOMAKE_OPTIONS = 1.9.6 foreign\n\n'
        emit += 'SUBDIRS = %s\n\n' % ' '.join(subdirs)
        emit += 'ACLOCAL_AMFLAGS = -I %s\n' % m4base
        emit = constants.nlconvert(emit)
        if type(emit) is bytes:
            emit = emit.decode(ENCS['default'])
        path = joinpath(self.testdir, 'Makefile.am')
        with codecs.open(path, 'wb', 'UTF-8') as file:
            file.write(emit)

        # Create configure.ac
        emit = string()
        emit += '# Process this file with autoconf '
        emit += 'to produce a configure script.\n'
        emit += 'AC_INIT([dummy], [0])\n'
        if auxdir != '.':
            emit += 'AC_CONFIG_AUX_DIR([%s])\n' % auxdir
        emit += 'AM_INIT_AUTOMAKE\n\n'
        emit += 'AC_CONFIG_HEADERS([config.h])\n\n'
        emit += 'AC_PROG_CC\n'
        emit += 'AC_PROG_INSTALL\n'
        emit += 'AC_PROG_MAKE_SET\n\n'
        emit += '# For autobuild.\n'
        emit += 'AC_CANONICAL_BUILD\n'
        emit += 'AC_CANONICAL_HOST\n\n'
        emit += 'm4_pattern_forbid([^gl_[A-Z]])dnl the gnulib macro namespace\n'
        emit += 'm4_pattern_allow([^gl_ES$])dnl a valid locale name\n'
        emit += 'm4_pattern_allow([^gl_LIBOBJS$])dnl a variable\n'
        emit += 'm4_pattern_allow([^gl_LTLIBOBJS$])dnl a variable\n\n'
        emit += 'gl_PROG_AR_RANLIB\n\n'
        if any_uses_subdirs:
            emit += 'AM_PROG_CC_C_O\n'
        snippets = list()
        for module in final_modules:
            if single_configure:
                solution = True
            else:  # if not single_configure
                solution = module.isNonTests()
            if solution:
                snippet = module.getAutoconfSnippet_Early()
                lines = [line for line in snippet.split('\n') if line.strip()]
                snippet = '\n'.join(lines)
                pattern = compiler(
                    'AC_REQUIRE\\(\\[([^()].*?)\\]\\)', re.S | re.M)
                snippet = pattern.sub('\\1', snippet)
                snippet = snippet.strip()
                snippets += [snippet]
        snippets = [snippet for snippet in snippets if snippet.strip()]
        emit += '%s\n' % '\n'.join(snippets)
        if libtool:
            emit += 'LT_INIT([win32-dll])\n'
            emit += 'LT_LANG([C++])\n'
            emit += 'AM_CONDITIONAL([GL_COND_LIBTOOL], [true])\n'
            emit += 'gl_cond_libtool=true\n'
        else:  # if not libtool
            emit += 'AM_CONDITIONAL([GL_COND_LIBTOOL], [false])\n'
            emit += 'gl_cond_libtool=false\n'
            emit += 'gl_libdeps=\n'
            emit += 'gl_ltlibdeps=\n'
        # Wrap the set of autoconf snippets into an autoconf macro that is then
        # invoked. This is needed because autoconf does not support AC_REQUIRE
        # at the top level:
        #   error: AC_REQUIRE(gt_CSHARPCOMP): cannot be used outside of an
        #     AC_DEFUN'd macro
        # but we want the AC_REQUIRE to have its normal meaning (provide one
        # expansion of the required macro before the current point, and only one
        # expansion total).
        emit += 'AC_DEFUN([gl_INIT], [\n'
        if auxdir != 'build-aux':
            replace_auxdir = True
        else:  # auxdir == 'build-aux'
            replace_auxdir = False
        emit += 'gl_m4_base=\'%s\'\n' % m4base
        emit += self.emiter.initmacro_start(macro_prefix)
        emit += 'gl_source_base=\'%s\'\n' % sourcebase
        if single_configure:
            emit += self.emiter.autoconfSnippets(main_modules, self.moduletable,
                                                 self.assistant, 0, False, False, False, replace_auxdir)
        else:  # if not single_configure
            emit += self.emiter.autoconfSnippets(modules, self.moduletable,
                                                 self.assistant, 1, False, False, False, replace_auxdir)
        emit += self.emiter.initmacro_end(macro_prefix)
        if single_configure:
            emit += '  gltests_libdeps=\n'
            emit += '  gltests_ltlibdeps=\n'
            emit += self.emiter.initmacro_start('%stests' % macro_prefix)
            emit += '  gl_source_base=\'%s\'\n' % testsbase
            # Define a tests witness macro.
            emit += '  %stests_WITNESS=IN_GNULIB_TESTS\n' % macro_prefix
            emit += '  AC_SUBST([%stests_WITNESS])\n' % macro_prefix
            emit += '  gl_module_indicator_condition=$%stests_WITNESS\n' % \
                macro_prefix
            emit += '  m4_pushdef([gl_MODULE_INDICATOR_CONDITION], '
            emit += '[$gl_module_indicator_condition])\n'
            snippets = self.emiter.autoconfSnippets(tests_modules, self.moduletable,
                                                    self.assistant, 1, True, False, False, replace_auxdir)
            emit += snippets.strip()
            emit += '  m4_popdef([gl_MODULE_INDICATOR_CONDITION])\n'
            emit += self.emiter.initmacro_end('%stests' % macro_prefix)
        # _LIBDEPS and _LTLIBDEPS variables are not needed if this library is
        # created using libtool, because libtool already handles the dependencies.
        if not libtool:
            libname_upper = libname.upper().replace('-', '_')
            emit += '  %s_LIBDEPS="$gl_libdeps"\n' % libname_upper
            emit += '  AC_SUBST([%s_LIBDEPS])\n' % libname_upper
            emit += '  %s_LTLIBDEPS="$gl_ltlibdeps"\n' % libname_upper
            emit += '  AC_SUBST([%s_LTLIBDEPS])\n' % libname_upper
        if single_configure and libtests:
            emit += '  LIBTESTS_LIBDEPS="$gltests_libdeps"\n'
            emit += '  AC_SUBST([LIBTESTS_LIBDEPS])\n'
        emit += '])\n'
        emit += self.emiter.initmacro_done(macro_prefix, sourcebase)
        if single_configure:
            emit += self.emiter.initmacro_done('%stests' %
                                               macro_prefix, testsbase)
        emit += '\ngl_INIT\n\n'
        if subdirs_with_configure_ac:
            if single_configure:
                emit += 'AC_CONFIG_SUBDIRS([%s])\n' % \
                    ' '.join(subdirs_with_configure_ac[:-1])
            else:  # if not single_configure
                emit += 'AC_CONFIG_SUBDIRS([%s])\n' % \
                    ' '.join(subdirs_with_configure_ac)
        makefiles = ['Makefile']
        for directory in subdirs:
            # For subdirs that have a configure.ac by their own, it's the subdir's
            # configure.ac which creates the subdir's Makefile.am, not this one.
            makefiles += [joinpath(directory, 'Makefile')]
        if not single_configure:
            makefiles = makefiles[:-1]
        emit += 'AC_CONFIG_FILES([%s])\n' % ' '.join(makefiles)
        emit += 'AC_OUTPUT\n'
        path = joinpath(self.testdir, 'configure.ac')
        with codecs.open(path, 'wb', 'UTF-8') as file:
            file.write(emit)

        # Create autogenerated files.
        # Do not use "${AUTORECONF} --force --install", because it may invoke
        # autopoint, which brings in older versions of some of our .m4 files.
        os.chdir(self.testdir)
        # gettext
        if isfile(joinpath(m4base, 'gettext.m4')):
            args = [UTILS['autopoint'], '--force']
            constants.execute(args, verbose)
            for src in os.listdir(m4base):
                src = joinpath(m4base, src)
                if src.endswith('.m4~'):
                    dest = src[:-1]
                    if isfile(dest):
                        os.remove(dest)
                    shutil.move(src, dest)
        # libtoolize
        if libtool:
            args = [UTILS['libtoolize'], '--copy']
            constants.execute(args, verbose)
        # aclocal
        args = [UTILS['aclocal'], '-I', m4base]
        constants.execute(args, verbose)
        if not isdir('build-aux'):
            os.mkdir('build-aux')
        # autoconf
        args = [UTILS['autoconf']]
        constants.execute(args, verbose)
        # autoheader
        args = [UTILS['autoheader']]
        constants.execute(args, verbose)
        # automake
        args = [UTILS['automake'], '--add-missing', '--copy']
        constants.execute(args, verbose)
        os.chdir(DIRS['cwd'])
        if inctests and not single_configure:
            # Do not use "${AUTORECONF} --force --install", because it may invoke
            # autopoint, which brings in older versions of some of our .m4 files.
            os.chdir(joinpath(self.testdir, testsbase))
            # gettext
            if isfile(joinpath(m4base, 'gettext.m4')):
                args = [UTILS['autopoint'], '--force']
                constants.execute(args, verbose)
                for src in os.listdir(m4base):
                    src = joinpath(m4base, src)
                    if src.endswith('.m4~'):
                        dest = src[:-1]
                        if isfile(dest):
                            os.remove(dest)
                        shutil.move(src, dest)
            # aclocal
            args = [UTILS['aclocal'], '-I', joinpath('..', m4base)]
            constants.execute(args, verbose)
            if not isdir(joinpath('../build-aux')):
                os.mkdir('../build-aux')
            # autoconf
            args = [UTILS['autoconf']]
            constants.execute(args, verbose)
            # autoheader
            args = [UTILS['autoheader']]
            constants.execute(args, verbose)
            # automake
            args = [UTILS['automake'], '--add-missing', '--copy']
            constants.execute(args, verbose)
            os.chdir(DIRS['cwd'])

        # Need to run configure and make once, to create built files that are to be
        # distributed (such as parse-datetime.c).
        path = joinpath(self.testdir, sourcebase, 'Makefile.am')
        with codecs.open(path, 'rb', 'UTF-8') as file:
            snippet = file.read()
        snippet = constants.remove_backslash_newline(snippet)
        cleaned_files = list()
        tests_cleaned_files = list()
        built_sources = list()
        tests_built_sources = list()
        distributed_built_sources = list()
        tests_distributed_built_sources = list()

        # Extract the value of "CLEANFILES += ..." and "MOSTLYCLEANFILES += ...".
        regex_find = list()
        pattern = compiler('^CLEANFILES[\t ]*\\+=(.*?)$', re.S | re.M)
        regex_find += pattern.findall(snippet)
        pattern = compiler('^MOSTLYCLEANFILES[\t ]*\\+=(.*?)$', re.S | re.M)
        regex_find += pattern.findall(snippet)
        regex_find = [line.strip() for line in regex_find if line.strip()]
        for part in regex_find:
            cleaned_files += \
                [line.strip() for line in part.split(' ') if line.strip()]

        # Extract the value of "BUILT_SOURCES += ...". Remove variable references
        # such $(FOO_H) because they don't refer to distributed files.
        regex_find = list()
        pattern = compiler('^BUILT_SOURCES[\t ]*\\+=(.*?)$', re.S | re.M)
        regex_find += pattern.findall(snippet)
        regex_find = [line.strip() for line in regex_find if line.strip()]
        for part in regex_find:
            built_sources += \
                [line.strip() for line in part.split(' ') if line.strip()]
        built_sources = [line for line in built_sources
                         if not bool(compiler('[$]\\([A-Za-z0-9_]*\\)$').findall(line))]
        distributed_built_sources = [file for file in built_sources
                                     if file not in cleaned_files]

        if inctests:
            # Likewise for built files in the $testsbase directory.
            path = joinpath(self.testdir, testsbase, 'Makefile.am')
            with codecs.open(path, 'rb', 'UTF-8') as file:
                snippet = file.read()
            snippet = constants.remove_backslash_newline(snippet)

            # Extract the value of "CLEANFILES += ..." and "MOSTLYCLEANFILES += ...".
            regex_find = list()
            pattern = compiler('^CLEANFILES[\t ]*\\+=(.*?)$', re.S | re.M)
            regex_find += pattern.findall(snippet)
            pattern = compiler(
                '^MOSTLYCLEANFILES[\t ]*\\+=(.*?)$', re.S | re.M)
            regex_find += pattern.findall(snippet)
            regex_find = [line.strip() for line in regex_find if line.strip()]
            for part in regex_find:
                tests_cleaned_files += \
                    [line.strip() for line in part.split(' ') if line.strip()]

            # Extract the value of "BUILT_SOURCES += ...". Remove variable references
            # such $(FOO_H) because they don't refer to distributed files.
            regex_find = list()
            tests_built_sources = list()
            pattern = compiler('^BUILT_SOURCES[\t ]*\\+=(.*?)$', re.S | re.M)
            regex_find += pattern.findall(snippet)
            regex_find = [line.strip() for line in regex_find if line.strip()]
            for part in regex_find:
                tests_built_sources += \
                    [line.strip() for line in part.split(' ') if line.strip()]
            tests_built_sources = [line for line in tests_built_sources
                                   if not bool(compiler('[$]\\([A-Za-z0-9_]*\\)$').findall(line))]
            tests_distributed_built_sources = [file for file in tests_built_sources
                                               if file not in cleaned_files]

        if distributed_built_sources or tests_distributed_built_sources:
            os.chdir(self.testdir)
            sp.call('./configure')
            if distributed_built_sources:
                os.chdir(sourcebase)
                with codecs.open('Makefile', 'ab', 'UTF-8') as file:
                    file.write('built_sources: $(BUILT_SOURCES)\n')
                args = [UTILS['make'],
                        'AUTOCONF=%s' % UTILS['autoconf'],
                        'AUTOHEADER=%s' % UTILS['autoheader'],
                        'ACLOCAL=%s' % UTILS['aclocal'],
                        'AUTOMAKE=%s' % UTILS['automake'],
                        'AUTORECONF=%s' % UTILS['autoreconf'],
                        'built_sources']
                sp.call(args)
                os.chdir('..')
            if tests_distributed_built_sources:
                os.chdir(testsbase)
                with codecs.open('Makefile', 'ab', 'UTF-8') as file:
                    file.write('built_sources: $(BUILT_SOURCES)\n')
                args = [UTILS['make'],
                        'AUTOCONF=%s' % UTILS['autoconf'],
                        'AUTOHEADER=%s' % UTILS['autoheader'],
                        'ACLOCAL=%s' % UTILS['aclocal'],
                        'AUTOMAKE=%s' % UTILS['automake'],
                        'AUTORECONF=%s' % UTILS['autoreconf'],
                        'built_sources']
                sp.call(args)
                os.chdir('..')
            args = [UTILS['make'],
                    'AUTOCONF=%s' % UTILS['autoconf'],
                    'AUTOHEADER=%s' % UTILS['autoheader'],
                    'ACLOCAL=%s' % UTILS['aclocal'],
                    'AUTOMAKE=%s' % UTILS['automake'],
                    'AUTORECONF=%s' % UTILS['autoreconf'],
                    'AUTOPOINT=%s' % UTILS['autopoint'],
                    'LIBTOOLIZE=%s' % UTILS['libtoolize'],
                    'distclean']
            sp.call(args)
        sp.call(['rm', '-rf', self.config['tempdir']], shell=False)


#===============================================================================
# Define GLMegaTestDir class
#===============================================================================
class GLMegaTestDir(object):
    '''GLMegaTestDir class is used to create a mega scratch package with the
    given modules one by one and all together.'''

    def __init__(self, config, megatestdir):
        '''GLMegaTestDir.__init__(config, megatestdir) -> GLMegaTestDir

        Create new GLTestDir instance.'''
        if type(config) is not GLConfig:
            raise(TypeError('config must be a GLConfig, not %s' %
                            type(config).__name__))
        if type(megatestdir) is bytes or type(megatestdir) is string:
            if type(megatestdir) is bytes:
                megatestdir = megatestdir.decode(ENCS['default'])
        self.config = config
        self.megatestdir = os.path.normpath(megatestdir)
        if not os.path.exists(self.megatestdir):
            try:  # Try to create directory
                os.mkdir(self.megatestdir)
            except Exception as error:
                raise(GLError(19, self.megatestdir))
        self.emiter = GLEmiter(self.config)
        self.filesystem = GLFileSystem(self.config)
        self.modulesystem = GLModuleSystem(self.config)
        self.moduletable = GLModuleTable(self.config)
        self.assistant = GLFileAssistant(self.config)
        self.makefiletable = GLMakefileTable(self.config)

    def execute(self):
        '''GLMegaTestDir.execute()

        Create a mega scratch package with the given modules one by one and all
        together.'''
        megasubdirs = list()
        modules = [self.modulesystem.find(m) for m in self.config['modules']]
        if not modules:
            modules = self.modulesystem.list()
            modules = [self.modulesystem.find(m) for m in modules]
        modules = sorted(set(modules))

        # First, all modules one by one.
        for module in modules:
            self.config.setModules([str(module)])
            #GLTestDir(self.config, self.megatestdir).execute()
            megasubdirs += [str(module)]

        # Then, all modules all together.
        # Except config-h, which breaks all modules which use HAVE_CONFIG_H.
        modules = [module for module in modules if str(module) != 'config-h']
        self.config.setModules([str(module) for module in modules])
        #GLTestDir(self.config, self.megatestdir).execute()
        megasubdirs += ['ALL']

        # Create autobuild.
        emit = string()
        repdict = dict()
        repdict['Jan'] = repdict['January'] = '01'
        repdict['Feb'] = repdict['February'] = '02'
        repdict['Mar'] = repdict['March'] = '03'
        repdict['Apr'] = repdict['April'] = '04'
        repdict['May'] = repdict['May'] = '05'
        repdict['Jun'] = repdict['June'] = '06'
        repdict['Jul'] = repdict['July'] = '07'
        repdict['Aug'] = repdict['August'] = '08'
        repdict['Sep'] = repdict['September'] = '09'
        repdict['Oct'] = repdict['October'] = '10'
        repdict['Nov'] = repdict['November'] = '11'
        repdict['Dec'] = repdict['December'] = '12'
        vc_witness = joinpath(DIRS['root'], '.git', 'refs', 'heads', 'master')
        mdate_sh = joinpath(DIRS['root'], 'build-aux', 'mdate-sh')
        args = ['sh', mdate_sh, vc_witness]
        cvsdate = sp.check_output(args).decode("UTF-8").strip()
        for key in repdict:
            if len(key) > 3:
                cvsdate = cvsdate.replace(key, repdict[key])
        for key in repdict:
            cvsdate = cvsdate.replace(key, repdict[key])
        cvsdate = ''.join(
            [date for date in cvsdate.split(' ') if date.strip()])
        cvsdate = '%s%s%s' % (cvsdate[4:], cvsdate[2:4], cvsdate[:2])
        emit += '#!/bin/sh\n'
        emit += 'CVSDATE=%s\n' % cvsdate
        emit += ': ${MAKE=make}\n'
        emit += 'test -d logs || mkdir logs\n'
        emit += 'for module in %s; do\n' % ' '.join(megasubdirs)
        emit += '  echo "Working on module $module..."\n'
        emit += '  safemodule=`echo $module | sed -e \'s|/|-|g\'`\n'
        emit += '  (echo "To: gnulib@autobuild.josefsson.org"\\\n'
        emit += '   echo\n'
        emit += '   set -x\n'
        emit += '   : autobuild project... $module\n'
        emit += '   : autobuild revision... cvs-$CVSDATE-000000\n'
        emit += '   : autobuild timestamp... `date "+%Y%m%d-%H%M%S"`\n'
        emit += '   : autobuild hostname... `hostname`\n'
        emit += '   cd $module && ./configure $CONFIGURE_OPTIONS && $MAKE'
        emit += ' && $MAKE check && $MAKE distclean\n'
        emit += '   echo rc=$?\n'
        emit += '  ) 2>&1 | { if test -n "$AUTOBUILD_SUBST"; then '
        emit += 'sed -e "$AUTOBUILD_SUBST"; else cat; fi; } > logs/$safemodule\n'
        emit += 'done\n'
        emit = constants.nlconvert(emit)
        if type(emit) is bytes:
            emit = emit.decode(ENCS['default'])
        path = joinpath(self.megatestdir, 'do-autobuild')
        with codecs.open(path, 'wb', 'UTF-8') as file:
            file.write(emit)

        # Create Makefile.am.
        emit = string()
        emit += '## Process this file with automake to produce Makefile.in.\n\n'
        emit += 'AUTOMAKE_OPTIONS = 1.9.6 foreign\n\n'
        emit += 'SUBDIRS = %s\n\n' % ' '.join(megasubdirs)
        emit += 'EXTRA_DIST = do-autobuild\n'
        emit = constants.nlconvert(emit)
        if type(emit) is bytes:
            emit = emit.decode(ENCS['default'])
        path = joinpath(self.megatestdir, 'Makefile.am')
        with codecs.open(path, 'wb', 'UTF-8') as file:
            file.write(emit)

        emit = string()
        emit += '# Process this file with autoconf '
        emit += 'to produce a configure script.\n'
        emit += 'AC_INIT([dummy], [0])\n\n'
        if auxdir != '.':
            emit += 'AC_CONFIG_AUX_DIR([%s])\n' % auxdir
        emit += 'AM_INIT_AUTOMAKE\n\n'
        emit += 'AC_PROG_MAKE_SET\n\n'
        emit += 'AC_CONFIG_SUBDIRS([%s])\n' % ' '.megasubdirs
        emit += 'AC_CONFIG_FILES([Makefile])\n'
        emit += 'AC_OUTPUT\n'
        emit = constants.nlconvert(emit)
        if type(emit) is bytes:
            emit = emit.decode(ENCS['default'])
        path = joinpath(self.megatestdir, 'Makefile.am')
        with codecs.open(path, 'wb', 'UTF-8') as file:
            file.write(emit)

        # Create autogenerated files.
        os.chdir(DIRS['cwd'])
        args = [UTILS['aclocal']]
        constants.execute(args, verbose)
        try:  # Try to make a directory
            if not isdir('build-aux'):
                os, mkdir('build-aux')
        except Exception as error:
            pass
        args = [UTILS['autoconf']]
        constants.execute(args, verbose)
        args = [UTILS['automake'], '--add-missing', '--copy']
        constants.execute(args, verbose)
        sp.call(['rm', '-rf', self.config['tempdir']], shell=False)



#!/usr/bin/env python
# Script that can read from a serial device and show timestamps.
#
# Copyright (C) 2009  Kevin O'Connor <kevin@koconnor.net>
#
# This file may be distributed under the terms of the GNU GPLv3 license.

# Usage:
#   scripts/readserial.py /dev/ttyUSB0 115200

import sys, os, time, select, optparse

from python23compat import as_bytes

# Reset time counter after this much idle time.
RESTARTINTERVAL = 60
# Number of bits in a transmitted byte - 8N1 is 1 start bit + 8 data
# bits + 1 stop bit.
BITSPERBYTE = 10

def calibrateserialwrite(outfile, byteadjust):
    # Build 4000 bytes of dummy data.
    data = "0123456789" * 4 + "012345678" + "\n"
    data = data * 80
    while 1:
        st = time.time()
        outfile.write(as_bytes(data))
        outfile.flush()
        et = time.time()
        sys.stdout.write(
            "Wrote %d - %.1fus per char (theory states %.1fus)\n" % (
                len(data), (et-st) / len(data) * 1000000, byteadjust * 1000000))
        sys.stdout.flush()
        time.sleep(3)

def calibrateserialread(infile, byteadjust):
    starttime = lasttime = 0
    totalchars = 0
    while 1:
        select.select([infile], [], [])
        d = infile.read(4096)
        curtime = time.time()
        if curtime - lasttime > 1.0:
            if starttime and totalchars:
                sys.stdout.write(
                    "Calibrating on %d bytes - %.1fus per char"
                    " (theory states %.1fus)\n" % (
                        totalchars,
                        float(lasttime - starttime) * 1000000 / totalchars,
                        byteadjust * 1000000))
            totalchars = 0
            starttime = curtime
        else:
            totalchars += len(d)
        lasttime = curtime

def readserial(infile, logfile, byteadjust):
    lasttime = 0
    while 1:
        # Read data
        try:
            res = select.select([infile, sys.stdin], [], [])
        except KeyboardInterrupt:
            sys.stdout.write("\n")
            return -1
        if sys.stdin in res[0]:
            # Got keyboard input - force reset on next serial input
            sys.stdin.read(1)
            lasttime = 0
            if len(res[0]) == 1:
                continue
        d = infile.read(4096)
        if not d:
            return 0
        datatime = time.time()

        datatime -= len(d) * byteadjust

        # Reset start time if no data for some time
        if datatime - lasttime > RESTARTINTERVAL:
            starttime = datatime
            charcount = 0
            isnewline = 1
            msg = "\n\n======= %s (adjust=%.1fus)\n" % (
                time.asctime(time.localtime(datatime)), byteadjust * 1000000)
            sys.stdout.write(msg)
            logfile.write(as_bytes(msg))
        lasttime = datatime

        # Translate unprintable chars; add timestamps
        out = as_bytes("")
        for c in d:
            if isnewline:
                delta = datatime - starttime - (charcount * byteadjust)
                out += "%06.3f: " % delta
                isnewline = 0
            oc = ord(c)
            charcount += 1
            datatime += byteadjust
            if oc == 0x0d:
                continue
            if oc == 0x00:
                out += "<00>\n"
                isnewline = 1
                continue
            if oc == 0x0a:
                out += "\n"
                isnewline = 1
                continue
            if oc < 0x20 or oc >= 0x7f and oc != 0x09:
                out += "<%02x>" % oc
                continue
            out += c

        if (sys.version_info > (3, 0)):
            sys.stdout.buffer.write(out)
        else:
            sys.stdout.write(out)
        sys.stdout.flush()
        logfile.write(out)
        logfile.flush()

def main():
    usage = "%prog [options] [<serialdevice> [<baud>]]"
    opts = optparse.OptionParser(usage)
    opts.add_option("-f", "--file",
                    action="store_false", dest="serial", default=True,
                    help="read from unix named pipe instead of serialdevice")
    opts.add_option("-n", "--no-adjust",
                    action="store_false", dest="adjustbaud", default=True,
                    help="don't adjust times by serial rate")
    opts.add_option("-c", "--calibrate-read",
                    action="store_true", dest="calibrate_read", default=False,
                    help="read from serial port to calibrate it")
    opts.add_option("-C", "--calibrate-write",
                    action="store_true", dest="calibrate_write", default=False,
                    help="write to serial port to calibrate it")
    opts.add_option("-t", "--time",
                    type="float", dest="time", default=None,
                    help="time to write one byte on serial port (in us)")
    options, args = opts.parse_args()
    serialport = 0
    baud = 115200
    if len(args) > 2:
        opts.error("Too many arguments")
    if len(args) > 0:
        serialport = args[0]
    if len(args) > 1:
        baud = int(args[1])
    byteadjust = float(BITSPERBYTE) / baud
    if options.time is not None:
        byteadjust = options.time / 1000000.0
    if not options.adjustbaud:
        byteadjust = 0.0

    if options.serial:
        # Read from serial port
        try:
            import serial
        except ImportError:
            print("""
Unable to find pyserial package ( http://pyserial.sourceforge.net/ ).
On Linux machines try: yum install pyserial
Or: apt-get install python-serial
""")
            sys.exit(1)
        ser = serial.Serial(serialport, baud, timeout=0)

    if options.calibrate_read:
        calibrateserialread(ser, byteadjust)
        return
    if options.calibrate_write:
        calibrateserialwrite(ser, byteadjust)
        return

    logname = time.strftime("seriallog-%Y%m%d_%H%M%S.log")
    f = open(logname, 'wb')
    if options.serial:
        readserial(ser, f, byteadjust)
    else:
        # Read from a pipe
        while 1:
            ser = os.fdopen(os.open(serialport, os.O_RDONLY|os.O_NONBLOCK), 'rb')
            res = readserial(ser, f, byteadjust)
            ser.close()
            if res < 0:
                break

if __name__ == '__main__':
    main()
